@@ -42,6 +42,7 @@ typedef struct tpm2_ecgen_ctx_st TPM2_ECGEN_CTX;
 
 struct tpm2_ecgen_ctx_st {
     const OSSL_CORE_HANDLE *core;
+    tpm2_semaphore_t esys_lock;
     ESYS_CONTEXT *esys_ctx;
     TPM2_CAPABILITY capability;
     TPM2_HANDLE parentHandle;
@@ -61,6 +62,8 @@ static OSSL_FUNC_keymgmt_load_fn tpm2_ec_keymgmt_load;
 static OSSL_FUNC_keymgmt_free_fn tpm2_ec_keymgmt_free;
 static OSSL_FUNC_keymgmt_get_params_fn tpm2_ec_keymgmt_get_params;
 static OSSL_FUNC_keymgmt_gettable_params_fn tpm2_ec_keymgmt_gettable_params;
+static OSSL_FUNC_keymgmt_set_params_fn tpm2_ec_keymgmt_set_params;
+static OSSL_FUNC_keymgmt_settable_params_fn tpm2_ec_keymgmt_settable_params;
 static OSSL_FUNC_keymgmt_has_fn tpm2_ec_keymgmt_has;
 static OSSL_FUNC_keymgmt_match_fn tpm2_ec_keymgmt_match;
 static OSSL_FUNC_keymgmt_import_fn tpm2_ec_keymgmt_import;
@@ -80,6 +83,7 @@ tpm2_ec_keymgmt_new(void *provctx)
     }
 
     pkey->core = cprov->core;
+    pkey->esys_lock = cprov->esys_lock;
     pkey->esys_ctx = cprov->esys_ctx;
     pkey->capability = cprov->capability;
     pkey->object = ESYS_TR_NONE;
@@ -103,6 +107,7 @@ tpm2_ec_keymgmt_gen_init(void *provctx, int selection, const OSSL_PARAM params[]
         return NULL;
 
     gen->core = cprov->core;
+    gen->esys_lock = cprov->esys_lock;
     gen->esys_ctx = cprov->esys_ctx;
     gen->capability = cprov->capability;
 
@@ -214,6 +219,7 @@ tpm2_ec_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
     }
 
     pkey->core = gen->core;
+    pkey->esys_lock = gen->esys_lock;
     pkey->esys_ctx = gen->esys_ctx;
     pkey->capability = gen->capability;
 
@@ -224,12 +230,13 @@ tpm2_ec_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
     /* load parent */
     if (gen->parentHandle && gen->parentHandle != TPM2_RH_OWNER) {
         DBG("EC GEN parent: persistent 0x%x\n", gen->parentHandle);
-        if (!tpm2_load_parent(pkey->core, pkey->esys_ctx,
+        if (!tpm2_load_parent(pkey->core, pkey->esys_lock, pkey->esys_ctx,
                               gen->parentHandle, &gen->parentAuth, &parent))
             goto error1;
     } else {
         DBG("EC GEN parent: primary 0x%x\n", TPM2_RH_OWNER);
-        if (!tpm2_build_primary(pkey->core, pkey->esys_ctx, pkey->capability.algorithms,
+        if (!tpm2_build_primary(pkey->core, pkey->esys_lock, pkey->esys_ctx,
+                                pkey->capability.algorithms,
                                 ESYS_TR_RH_OWNER, &gen->parentAuth, &parent))
             goto error1;
     }
@@ -237,6 +244,8 @@ tpm2_ec_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
     TPM2B_DATA outside_info = { .size = 0 };
     TPML_PCR_SELECTION creation_pcr = { .count = 0 };
 
+    if (!tpm2_semaphore_lock(gen->esys_lock))
+        return 0;
     /* older TPM2 chips do not support Esys_CreateLoaded */
     r = Esys_Create(gen->esys_ctx, parent,
                     ESYS_TR_PASSWORD, ESYS_TR_NONE, ESYS_TR_NONE,
@@ -255,15 +264,21 @@ tpm2_ec_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
     free(keyPrivate);
     TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_CREATE_KEY, goto error1);
 
-final:
     if (gen->parentHandle && gen->parentHandle != TPM2_RH_OWNER)
         Esys_TR_Close(gen->esys_ctx, &parent);
     else
         Esys_FlushContext(gen->esys_ctx, parent);
 
-    if (r == TSS2_RC_SUCCESS)
-        return pkey;
+    if (gen->inSensitive.sensitive.userAuth.size > 0) {
+        r = Esys_TR_SetAuth(gen->esys_ctx, pkey->object, &gen->inSensitive.sensitive.userAuth);
+        TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_LOAD_KEY, goto error2);
+    }
+    tpm2_semaphore_unlock(gen->esys_lock);
+    return pkey;
+error2:
+    Esys_FlushContext(gen->esys_ctx, pkey->object);
 error1:
+    tpm2_semaphore_unlock(gen->esys_lock);
     OPENSSL_clear_free(pkey, sizeof(TPM2_PKEY));
     return NULL;
 }
@@ -308,9 +323,9 @@ tpm2_ec_keymgmt_free(void *keydata)
 
     if (pkey->object != ESYS_TR_NONE) {
         if (pkey->data.privatetype == KEY_TYPE_HANDLE)
-            Esys_TR_Close(pkey->esys_ctx, &pkey->object);
+            tpm2_esys_tr_close(pkey->esys_lock, pkey->esys_ctx, &pkey->object);
         else
-            Esys_FlushContext(pkey->esys_ctx, pkey->object);
+            tpm2_esys_flush_context(pkey->esys_lock, pkey->esys_ctx, pkey->object);
     }
 
     OPENSSL_clear_free(pkey, sizeof(TPM2_PKEY));
@@ -343,8 +358,11 @@ tpm2_ec_keymgmt_get_params(void *keydata, OSSL_PARAM params[])
         return 1;
     TRACE_PARAMS("EC GET_PARAMS", params);
 
+    if (!tpm2_semaphore_lock(pkey->esys_lock))
+        return 0;
     r = Esys_ECC_Parameters(pkey->esys_ctx, ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
             TPM2_PKEY_EC_CURVE(pkey), &details);
+    tpm2_semaphore_unlock(pkey->esys_lock);
     TPM2_CHECK_RC(pkey->core, r, TPM2_ERR_UNKNOWN_ALGORITHM, return 0);
 
     p = OSSL_PARAM_locate(params, OSSL_PKEY_PARAM_GROUP_NAME);
@@ -421,6 +439,10 @@ tpm2_ec_keymgmt_get_params(void *keydata, OSSL_PARAM params[])
     p = OSSL_PARAM_locate(params, OSSL_PKEY_PARAM_EC_PUB_Y);
     if (p != NULL && !tpm2_param_set_BN_from_buffer(p, pkey->data.pub.publicArea.unique.ecc.y))
         goto error;
+    p = OSSL_PARAM_locate(params, OSSL_PKEY_PARAM_ENCODED_PUBLIC_KEY);
+    if (p != NULL && !tpm2_param_set_ecc_point(p, &pkey->data.pub.publicArea.unique.ecc.x,
+                                                  &pkey->data.pub.publicArea.unique.ecc.y))
+        goto error;
     free(details);
     return 1;
 error:
@@ -449,12 +471,45 @@ tpm2_ec_keymgmt_gettable_params(void *provctx)
         OSSL_PARAM_octet_string(OSSL_PKEY_PARAM_PUB_KEY, NULL, 0),
         OSSL_PARAM_BN(OSSL_PKEY_PARAM_EC_PUB_X, NULL, 0),
         OSSL_PARAM_BN(OSSL_PKEY_PARAM_EC_PUB_Y, NULL, 0),
+        OSSL_PARAM_octet_string(OSSL_PKEY_PARAM_ENCODED_PUBLIC_KEY, NULL, 0),
         OSSL_PARAM_END
     };
 
     return gettable;
 }
 
+static int
+tpm2_ec_keymgmt_set_params(void *keydata, const OSSL_PARAM params[])
+{
+    TPM2_PKEY *pkey = (TPM2_PKEY *)keydata;
+    const OSSL_PARAM *p;
+
+    if (params == NULL)
+        return 1;
+    TRACE_PARAMS("EC SET_PARAMS", params);
+
+    p = OSSL_PARAM_locate_const(params, OSSL_PKEY_PARAM_ENCODED_PUBLIC_KEY);
+    if (p != NULL) {
+        if (p->data_type != OSSL_PARAM_OCTET_STRING ||
+                !tpm2_buffer_to_ecc_point(tpm2_ecc_curve_to_nid(TPM2_PKEY_EC_CURVE(pkey)),
+                        p->data, p->data_size, &pkey->data.pub.publicArea.unique.ecc))
+            return 0;
+    }
+
+    return 1;
+}
+
+static const OSSL_PARAM *
+tpm2_ec_keymgmt_settable_params(void *provctx)
+{
+    static OSSL_PARAM settable[] = {
+        OSSL_PARAM_octet_string(OSSL_PKEY_PARAM_ENCODED_PUBLIC_KEY, NULL, 0),
+        OSSL_PARAM_END
+    };
+
+    return settable;
+}
+
 static const char *
 tpm2_ec_keymgmt_query_operation_name(int operation_id)
 {
@@ -467,6 +522,9 @@ tpm2_ec_keymgmt_query_operation_name(int operation_id)
     return NULL;
 }
 
+#define EC_POSSIBLE_SELECTIONS \
+    (OSSL_KEYMGMT_SELECT_KEYPAIR | OSSL_KEYMGMT_SELECT_ALL_PARAMETERS)
+
 static int
 tpm2_ec_keymgmt_has(const void *keydata, int selection)
 {
@@ -474,16 +532,21 @@ tpm2_ec_keymgmt_has(const void *keydata, int selection)
     int ok = 1;
 
     DBG("EC HAS 0x%x\n", selection);
-    if (pkey != NULL) {
-        /* although not exportable we may have the the private portion */
-        if ((selection & OSSL_KEYMGMT_SELECT_PRIVATE_KEY) != 0)
-            ok = ok && (pkey->data.privatetype != KEY_TYPE_NONE);
-        if ((selection & OSSL_KEYMGMT_SELECT_PUBLIC_KEY) != 0)
-            ok = ok && (pkey->data.pub.publicArea.unique.ecc.x.size > 0)
-                    && (pkey->data.pub.publicArea.unique.ecc.y.size > 0);
-        if ((selection & OSSL_KEYMGMT_SELECT_DOMAIN_PARAMETERS) != 0)
-            ok = ok && (TPM2_PKEY_EC_CURVE(pkey) != 0);
-    }
+
+    if (pkey == NULL)
+        return 0;
+    if ((selection & EC_POSSIBLE_SELECTIONS) == 0)
+        return 1; /* the selection is not missing */
+
+    /* although not exportable we may have the the private portion */
+    if ((selection & OSSL_KEYMGMT_SELECT_PRIVATE_KEY) != 0)
+        ok = ok && (pkey->data.privatetype != KEY_TYPE_NONE);
+    if ((selection & OSSL_KEYMGMT_SELECT_PUBLIC_KEY) != 0)
+        ok = ok && (pkey->data.pub.publicArea.unique.ecc.x.size > 0)
+                && (pkey->data.pub.publicArea.unique.ecc.y.size > 0);
+    if ((selection & OSSL_KEYMGMT_SELECT_DOMAIN_PARAMETERS) != 0)
+        ok = ok && (TPM2_PKEY_EC_CURVE(pkey) != 0);
+
     return ok;
 }
 
@@ -565,6 +628,9 @@ tpm2_ec_keymgmt_export(void *keydata, int selection,
     if (pkey == NULL)
         return 0;
 
+    if ((selection & OSSL_KEYMGMT_SELECT_PRIVATE_KEY) != 0)
+        return 0;
+
     curve_nid = tpm2_ecc_curve_to_nid(TPM2_PKEY_EC_CURVE(pkey));
 
     pubsize = tpm2_ecc_point_to_uncompressed(
@@ -612,6 +678,8 @@ const OSSL_DISPATCH tpm2_ec_keymgmt_functions[] = {
     { OSSL_FUNC_KEYMGMT_FREE, (void(*)(void))tpm2_ec_keymgmt_free },
     { OSSL_FUNC_KEYMGMT_GET_PARAMS, (void(*)(void))tpm2_ec_keymgmt_get_params },
     { OSSL_FUNC_KEYMGMT_GETTABLE_PARAMS, (void(*)(void))tpm2_ec_keymgmt_gettable_params },
+    { OSSL_FUNC_KEYMGMT_SET_PARAMS, (void(*)(void))tpm2_ec_keymgmt_set_params },
+    { OSSL_FUNC_KEYMGMT_SETTABLE_PARAMS, (void(*)(void))tpm2_ec_keymgmt_settable_params },
     { OSSL_FUNC_KEYMGMT_QUERY_OPERATION_NAME, (void(*)(void))tpm2_ec_keymgmt_query_operation_name },
     { OSSL_FUNC_KEYMGMT_HAS, (void(*)(void))tpm2_ec_keymgmt_has },
     { OSSL_FUNC_KEYMGMT_MATCH, (void(*)(void))tpm2_ec_keymgmt_match },
