@@ -42,6 +42,7 @@ typedef struct tpm2_rsagen_ctx_st TPM2_RSAGEN_CTX;
 
 struct tpm2_rsagen_ctx_st {
     const OSSL_CORE_HANDLE *core;
+    tpm2_semaphore_t esys_lock;
     ESYS_CONTEXT *esys_ctx;
     TPM2_CAPABILITY capability;
     TPM2_HANDLE parentHandle;
@@ -82,6 +83,7 @@ tpm2_rsa_keymgmt_new(void *provctx)
     }
 
     pkey->core = cprov->core;
+    pkey->esys_lock = cprov->esys_lock;
     pkey->esys_ctx = cprov->esys_ctx;
     pkey->capability = cprov->capability;
     pkey->object = ESYS_TR_NONE;
@@ -103,6 +105,7 @@ tpm2_create_rsagen_ctx(void *provctx)
         return NULL;
 
     gen->core = cprov->core;
+    gen->esys_lock = cprov->esys_lock;
     gen->esys_ctx = cprov->esys_ctx;
     gen->capability = cprov->capability;
     return gen;
@@ -258,6 +261,7 @@ tpm2_rsa_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
     }
 
     pkey->core = gen->core;
+    pkey->esys_lock = gen->esys_lock;
     pkey->esys_ctx = gen->esys_ctx;
     pkey->capability = gen->capability;
 
@@ -268,12 +272,13 @@ tpm2_rsa_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
     /* load parent */
     if (gen->parentHandle && gen->parentHandle != TPM2_RH_OWNER) {
         DBG("RSA GEN parent: persistent 0x%x\n", gen->parentHandle);
-        if (!tpm2_load_parent(pkey->core, pkey->esys_ctx,
+        if (!tpm2_load_parent(pkey->core, pkey->esys_lock, pkey->esys_ctx,
                               gen->parentHandle, &gen->parentAuth, &parent))
             goto error1;
     } else {
         DBG("RSA GEN parent: primary 0x%x\n", TPM2_RH_OWNER);
-        if (!tpm2_build_primary(pkey->core, pkey->esys_ctx, pkey->capability.algorithms,
+        if (!tpm2_build_primary(pkey->core, pkey->esys_lock, pkey->esys_ctx,
+                                pkey->capability.algorithms,
                                 ESYS_TR_RH_OWNER, &gen->parentAuth, &parent))
             goto error1;
     }
@@ -281,12 +286,14 @@ tpm2_rsa_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
     TPM2B_DATA outside_info = { .size = 0 };
     TPML_PCR_SELECTION creation_pcr = { .count = 0 };
 
+    if (!tpm2_semaphore_lock(gen->esys_lock))
+        goto error1;
     /* older TPM2 chips do not support Esys_CreateLoaded */
     r = Esys_Create(gen->esys_ctx, parent,
                     ESYS_TR_PASSWORD, ESYS_TR_NONE, ESYS_TR_NONE,
                     &gen->inSensitive, &gen->inPublic, &outside_info, &creation_pcr,
                     &keyPrivate, &keyPublic, NULL, NULL, NULL);
-    TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_CREATE_KEY, goto error1);
+    TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_CREATE_KEY, goto error2);
 
     pkey->data.pub = *keyPublic;
     pkey->data.privatetype = KEY_TYPE_BLOB;
@@ -297,16 +304,23 @@ tpm2_rsa_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
                   keyPrivate, keyPublic, &pkey->object);
     free(keyPublic);
     free(keyPrivate);
-    TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_CREATE_KEY, goto error1);
+    TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_CREATE_KEY, goto error2);
 
-final:
     if (gen->parentHandle && gen->parentHandle != TPM2_RH_OWNER)
         Esys_TR_Close(gen->esys_ctx, &parent);
     else
         Esys_FlushContext(gen->esys_ctx, parent);
 
-    if (r == TSS2_RC_SUCCESS)
-        return pkey;
+    if (gen->inSensitive.sensitive.userAuth.size > 0) {
+        r = Esys_TR_SetAuth(gen->esys_ctx, pkey->object, &gen->inSensitive.sensitive.userAuth);
+        TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_LOAD_KEY, goto error3);
+    }
+    tpm2_semaphore_unlock(gen->esys_lock);
+    return pkey;
+error3:
+    Esys_FlushContext(gen->esys_ctx, pkey->object);
+error2:
+    tpm2_semaphore_unlock(gen->esys_lock);
 error1:
     OPENSSL_clear_free(pkey, sizeof(TPM2_PKEY));
     return NULL;
@@ -352,9 +366,9 @@ tpm2_rsa_keymgmt_free(void *keydata)
 
     if (pkey->object != ESYS_TR_NONE) {
         if (pkey->data.privatetype == KEY_TYPE_HANDLE)
-            Esys_TR_Close(pkey->esys_ctx, &pkey->object);
+            tpm2_esys_tr_close(pkey->esys_lock, pkey->esys_ctx, &pkey->object);
         else
-            Esys_FlushContext(pkey->esys_ctx, pkey->object);
+            tpm2_esys_flush_context(pkey->esys_lock, pkey->esys_ctx, pkey->object);
     }
 
     OPENSSL_clear_free(pkey, sizeof(TPM2_PKEY));
@@ -396,7 +410,8 @@ tpm2_rsa_keymgmt_get_params(void *keydata, OSSL_PARAM params[])
             return 0;
     }
     p = OSSL_PARAM_locate(params, OSSL_PKEY_PARAM_MAX_SIZE);
-    if (p != NULL && !OSSL_PARAM_set_int(p, TPM2_MAX_RSA_KEY_BYTES))
+    if (p != NULL && !OSSL_PARAM_set_int(p,
+                            tpm2_rsa_size(&pkey->data.pub.publicArea.unique.rsa)))
         return 0;
 
     if (TPM2_PKEY_RSA_SCHEME(pkey) != TPM2_ALG_NULL) {
@@ -444,6 +459,9 @@ tpm2_rsa_keymgmt_query_operation_name(int operation_id)
     return "RSA";
 }
 
+#define RSA_POSSIBLE_SELECTIONS \
+    (OSSL_KEYMGMT_SELECT_KEYPAIR | OSSL_KEYMGMT_SELECT_OTHER_PARAMETERS)
+
 static int
 tpm2_rsa_keymgmt_has(const void *keydata, int selection)
 {
@@ -451,13 +469,18 @@ tpm2_rsa_keymgmt_has(const void *keydata, int selection)
     int ok = 1;
 
     DBG("RSA HAS %x\n", selection);
-    if (pkey != NULL) {
-        /* although not exportable we may have the the private portion */
-        if ((selection & OSSL_KEYMGMT_SELECT_PRIVATE_KEY) != 0)
-            ok = ok && (pkey->data.privatetype != KEY_TYPE_NONE);
-        if ((selection & OSSL_KEYMGMT_SELECT_PUBLIC_KEY) != 0)
-            ok = ok && (pkey->data.pub.publicArea.unique.rsa.size > 0);
-    }
+
+    if (pkey == NULL)
+        return 0;
+    if ((selection & RSA_POSSIBLE_SELECTIONS) == 0)
+        return 1; /* the selection is not missing */
+
+    /* although not exportable we may have the the private portion */
+    if ((selection & OSSL_KEYMGMT_SELECT_PRIVATE_KEY) != 0)
+        ok = ok && (pkey->data.privatetype != KEY_TYPE_NONE);
+    if ((selection & OSSL_KEYMGMT_SELECT_PUBLIC_KEY) != 0)
+        ok = ok && (pkey->data.pub.publicArea.unique.rsa.size > 0);
+
     return ok;
 }
 
@@ -550,6 +573,9 @@ tpm2_rsa_keymgmt_export(void *keydata, int selection,
     if (pkey == NULL)
         return 0;
 
+    if ((selection & OSSL_KEYMGMT_SELECT_PRIVATE_KEY) != 0)
+        return 0;
+
     OSSL_PARAM params[3], *p = params;
 #if !defined(WORDS_BIGENDIAN)
     unsigned char *n = NULL;
