@@ -30,6 +30,7 @@ static const hash_names_t hashes[] = {
     { "SHA512", TPM2_ALG_SHA512 },
     { "SHA-512", TPM2_ALG_SHA512 },
     { "SHA2-512", TPM2_ALG_SHA512 },
+    { "SM3", TPM2_ALG_SM3_256 },
     { NULL, TPM2_ALG_ERROR }
 };
 
@@ -115,6 +116,20 @@ tpm2_rsa_scheme_alg_to_name(const TPMI_ALG_RSA_SCHEME alg)
     return NULL;
 }
 
+int
+tpm2_rsa_size(const TPM2B_PUBLIC_KEY_RSA *rsa)
+{
+    BIGNUM *bn;
+    int ret = TPM2_MAX_RSA_KEY_BYTES;
+
+    if (rsa && (bn = BN_bin2bn(rsa->buffer, rsa->size, NULL))) {
+        ret = BN_num_bytes(bn);
+        BN_free(bn);
+    }
+
+    return ret;
+}
+
 typedef struct {
     int nid;
     TPM2_ECC_CURVE curve;
