@@ -4,6 +4,7 @@
 #include <openssl/params.h>
 #include <openssl/core_dispatch.h>
 #include <openssl/core_names.h>
+#include <openssl/prov_ssl.h>
 
 #include <tss2/tss2_tctildr.h>
 
@@ -58,15 +59,15 @@ typedef struct {
 
 static const OSSL_ALGORITHM *
 tpm2_operation(const TPM2_CAPABILITY *caps,
-               const TPM2_ALGORITHM *algs, size_t count)
+               const TPM2_ALGORITHM *algs, size_t algs_size)
 {
     OSSL_ALGORITHM *res;
     int i, j = 0;
 
-    if ((res = OPENSSL_malloc(count * sizeof(OSSL_ALGORITHM))) == NULL)
+    if ((res = OPENSSL_malloc(algs_size * sizeof(OSSL_ALGORITHM))) == NULL)
         return NULL;
 
-    for (i = 0; i < count && algs[i].algs != NULL; i++) {
+    for (i = 0; algs[i].algs != NULL; i++) {
         /* retrieve the implementation,
          * or NULL when the current TPM does not support this algorithm */
         if ((res[j].implementation = algs[i].dispatch(caps)) == NULL)
@@ -149,21 +150,27 @@ static const OSSL_ALGORITHM tpm2_rands[] = {
     { NULL, NULL, NULL }
 };
 
+#define PROV_NAMES_EC "EC:id-ecPublicKey:1.2.840.10045.2.1"
+#define PROV_NAMES_ECDH "ECDH"
+#define PROV_NAMES_ECDSA "ECDSA"
+#define PROV_NAMES_RSA "RSA:rsaEncryption:1.2.840.113549.1.1.1"
+#define PROV_NAMES_RSA_PSS "RSA-PSS:RSASSA-PSS:1.2.840.113549.1.1.10"
+
 extern tpm2_dispatch_t tpm2_rsa_keymgmt_dispatch;
 extern tpm2_dispatch_t tpm2_rsapss_keymgmt_dispatch;
 extern tpm2_dispatch_t tpm2_ec_keymgmt_dispatch;
 
 static const TPM2_ALGORITHM tpm2_keymgmts[] = {
-    { "RSA:rsaEncryption", "provider=tpm2", &tpm2_rsa_keymgmt_dispatch },
-    { "RSA-PSS:RSASSA-PSS", "provider=tpm2", &tpm2_rsapss_keymgmt_dispatch },
-    { "EC:id-ecPublicKey", "provider=tpm2", &tpm2_ec_keymgmt_dispatch },
+    { PROV_NAMES_RSA, "provider=tpm2", &tpm2_rsa_keymgmt_dispatch },
+    { PROV_NAMES_RSA_PSS, "provider=tpm2", &tpm2_rsapss_keymgmt_dispatch },
+    { PROV_NAMES_EC, "provider=tpm2", &tpm2_ec_keymgmt_dispatch },
     { NULL, NULL, NULL }
 };
 
 extern const OSSL_DISPATCH tpm2_ecdh_keyexch_functions[];
 
 static const OSSL_ALGORITHM tpm2_keyexchs[] = {
-    { "ECDH", "provider=tpm2", tpm2_ecdh_keyexch_functions },
+    { PROV_NAMES_ECDH, "provider=tpm2", tpm2_ecdh_keyexch_functions },
     { NULL, NULL, NULL }
 };
 
@@ -171,15 +178,15 @@ extern const OSSL_DISPATCH tpm2_rsa_signature_functions[];
 extern const OSSL_DISPATCH tpm2_ecdsa_signature_functions[];
 
 static const OSSL_ALGORITHM tpm2_signatures[] = {
-    { "RSA:rsaEncryption", TPM2_PROPS(signature), tpm2_rsa_signature_functions },
-    { "ECDSA", TPM2_PROPS(signature), tpm2_ecdsa_signature_functions },
+    { PROV_NAMES_RSA, TPM2_PROPS(signature), tpm2_rsa_signature_functions },
+    { PROV_NAMES_ECDSA, TPM2_PROPS(signature), tpm2_ecdsa_signature_functions },
     { NULL, NULL, NULL }
 };
 
 extern const OSSL_DISPATCH tpm2_rsa_asymcipher_functions[];
 
 static const OSSL_ALGORITHM tpm2_asymciphers[] = {
-    { "RSA:rsaEncryption", "provider=tpm2", tpm2_rsa_asymcipher_functions },
+    { PROV_NAMES_RSA, "provider=tpm2", tpm2_rsa_asymcipher_functions },
     { NULL, NULL, NULL }
 };
 
@@ -227,16 +234,16 @@ extern const OSSL_DISPATCH tpm2_tss_to_ec_decoder_functions[];
 
 static const OSSL_ALGORITHM tpm2_decoders[] = {
     { "DER", "provider=tpm2,input=pem", tpm2_der_decoder_functions },
-    { "RSA:rsaEncryption", "provider=tpm2,input=der,structure=TSS2", tpm2_tss_to_rsa_decoder_functions },
-    { "EC:id-ecPublicKey", "provider=tpm2,input=der,structure=TSS2", tpm2_tss_to_ec_decoder_functions },
+    { "RSA", "provider=tpm2,input=der,structure=TSS2", tpm2_tss_to_rsa_decoder_functions },
+    { "EC", "provider=tpm2,input=der,structure=TSS2", tpm2_tss_to_ec_decoder_functions },
     { NULL, NULL, NULL }
 };
 
-extern const OSSL_DISPATCH tpm2_object_store_functions[];
+extern const OSSL_DISPATCH tpm2_handle_store_functions[];
 
 static const OSSL_ALGORITHM tpm2_stores[] = {
-    { "object", TPM2_PROPS(store), tpm2_object_store_functions },
-    { "handle", TPM2_PROPS(store), tpm2_object_store_functions },
+    { "handle", TPM2_PROPS(store), tpm2_handle_store_functions },
+    { "object", TPM2_PROPS(store), tpm2_handle_store_functions },
     { NULL, NULL, NULL }
 };
 
@@ -333,9 +340,13 @@ tpm2_self_test(void *provctx)
     TSS2_RC r;
 
     DBG("PROVIDER SELFTEST\n");
+
+    if (!tpm2_semaphore_lock(cprov->esys_lock))
+        return 0;
     r = Esys_SelfTest(cprov->esys_ctx,
                       ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                       TPM2_YES);
+    tpm2_semaphore_unlock(cprov->esys_lock);
 
     return r == TPM2_RC_SUCCESS;
 }
@@ -351,6 +362,7 @@ tpm2_teardown(void *provctx)
     free(cprov->capability.properties);
     free(cprov->capability.algorithms);
     free(cprov->capability.commands);
+    tpm2_semaphore_free(cprov->esys_lock);
     OSSL_LIB_CTX_free(cprov->libctx);
 
     r = Esys_GetTcti(cprov->esys_ctx, &tcti_ctx);
@@ -362,6 +374,86 @@ tpm2_teardown(void *provctx)
     OPENSSL_clear_free(cprov, sizeof(TPM2_PROVIDER_CTX));
 }
 
+typedef struct tls_group_constants_st {
+    unsigned int group_id;   /* Group ID */
+    unsigned int secbits;    /* Bits of security */
+    int mintls;              /* Minimum TLS version, -1 unsupported */
+    int maxtls;              /* Maximum TLS version (or 0 for undefined) */
+    int mindtls;             /* Minimum DTLS version, -1 unsupported */
+    int maxdtls;             /* Maximum DTLS version (or 0 for undefined) */
+} TLS_GROUP_CONSTANTS;
+
+#define TLS_GROUP_ID_secp192r1 19
+#define TLS_GROUP_ID_secp224r1 21
+#define TLS_GROUP_ID_secp256r1 23
+#define TLS_GROUP_ID_secp384r1 24
+#define TLS_GROUP_ID_secp521r1 25
+
+static const TLS_GROUP_CONSTANTS tls_group_list[] = {
+    { TLS_GROUP_ID_secp192r1, 80, TLS1_VERSION, TLS1_2_VERSION,
+      DTLS1_VERSION, DTLS1_2_VERSION },
+    { TLS_GROUP_ID_secp224r1, 112, TLS1_VERSION, TLS1_2_VERSION,
+      DTLS1_VERSION, DTLS1_2_VERSION },
+    { TLS_GROUP_ID_secp256r1, 128, TLS1_VERSION, 0, DTLS1_VERSION, 0 },
+    { TLS_GROUP_ID_secp384r1, 192, TLS1_VERSION, 0, DTLS1_VERSION, 0 },
+    { TLS_GROUP_ID_secp521r1, 256, TLS1_VERSION, 0, DTLS1_VERSION, 0 },
+};
+
+#define TLS_GROUP_ENTRY(tlsname, realname, algorithm, idx) \
+    { \
+        OSSL_PARAM_utf8_string(OSSL_CAPABILITY_TLS_GROUP_NAME, \
+                               tlsname, \
+                               sizeof(tlsname)), \
+        OSSL_PARAM_utf8_string(OSSL_CAPABILITY_TLS_GROUP_NAME_INTERNAL, \
+                               realname, \
+                               sizeof(realname)), \
+        OSSL_PARAM_utf8_string(OSSL_CAPABILITY_TLS_GROUP_ALG, \
+                               algorithm, \
+                               sizeof(algorithm)), \
+        OSSL_PARAM_uint(OSSL_CAPABILITY_TLS_GROUP_ID, \
+                        (unsigned int *)&tls_group_list[idx].group_id), \
+        OSSL_PARAM_uint(OSSL_CAPABILITY_TLS_GROUP_SECURITY_BITS, \
+                        (unsigned int *)&tls_group_list[idx].secbits), \
+        OSSL_PARAM_int(OSSL_CAPABILITY_TLS_GROUP_MIN_TLS, \
+                       (int *)&tls_group_list[idx].mintls),     \
+        OSSL_PARAM_int(OSSL_CAPABILITY_TLS_GROUP_MAX_TLS, \
+                       (int *)&tls_group_list[idx].maxtls), \
+        OSSL_PARAM_int(OSSL_CAPABILITY_TLS_GROUP_MIN_DTLS, \
+                       (int *)&tls_group_list[idx].mindtls), \
+        OSSL_PARAM_int(OSSL_CAPABILITY_TLS_GROUP_MAX_DTLS, \
+                       (int *)&tls_group_list[idx].maxdtls), \
+        OSSL_PARAM_END \
+    }
+
+static const OSSL_PARAM param_tls_group_list[][10] = {
+    TLS_GROUP_ENTRY("secp192r1", "prime192v1", "EC", 0),
+    TLS_GROUP_ENTRY("P-192", "prime192v1", "EC", 0), /* Alias of previous */
+    TLS_GROUP_ENTRY("secp224r1", "secp224r1", "EC", 1),
+    TLS_GROUP_ENTRY("P-224", "secp224r1", "EC", 1), /* Alias of previous */
+    TLS_GROUP_ENTRY("secp256r1", "prime256v1", "EC", 2),
+    TLS_GROUP_ENTRY("P-256", "prime256v1", "EC", 2), /* Alias of previous */
+    TLS_GROUP_ENTRY("secp384r1", "secp384r1", "EC", 3),
+    TLS_GROUP_ENTRY("P-384", "secp384r1", "EC", 3), /* Alias of previous */
+    TLS_GROUP_ENTRY("secp521r1", "secp521r1", "EC", 4),
+    TLS_GROUP_ENTRY("P-521", "secp521r1", "EC", 4), /* Alias of above */
+};
+
+static int tpm2_get_capabilities(void *provctx, const char *capability,
+                                 OSSL_CALLBACK *cb, void *arg)
+{
+    if (OPENSSL_strcasecmp(capability, "TLS-GROUP") == 0) {
+        size_t i;
+
+        for (i = 0; i < NELEMS(param_tls_group_list); i++)
+            if (!cb(param_tls_group_list[i], arg))
+                return 0;
+
+        return 1;
+    }
+
+    return 0;
+}
+
 static const OSSL_DISPATCH tpm2_dispatch_table[] = {
     { OSSL_FUNC_PROVIDER_GETTABLE_PARAMS, (void (*)(void))tpm2_gettable_params },
     { OSSL_FUNC_PROVIDER_GET_PARAMS, (void (*)(void))tpm2_get_params },
@@ -370,6 +462,7 @@ static const OSSL_DISPATCH tpm2_dispatch_table[] = {
     { OSSL_FUNC_PROVIDER_GET_REASON_STRINGS, (void (*)(void))tpm2_get_reason_strings },
     { OSSL_FUNC_PROVIDER_SELF_TEST, (void (*)(void))tpm2_self_test },
     { OSSL_FUNC_PROVIDER_TEARDOWN, (void (*)(void))tpm2_teardown },
+    { OSSL_FUNC_PROVIDER_GET_CAPABILITIES, (void (*)(void))tpm2_get_capabilities },
     { 0, NULL }
 };
 
@@ -412,28 +505,33 @@ OSSL_provider_init(const OSSL_CORE_HANDLE *handle,
     r = Esys_Initialize(&cprov->esys_ctx, tcti_ctx, NULL);
     TPM2_CHECK_RC(cprov->core, r, TPM2_ERR_CANNOT_CONNECT, goto err2);
 
+    if ((cprov->esys_lock = tpm2_semaphore_new()) == NULL)
+        goto err3;
+
     r = Esys_GetCapability(cprov->esys_ctx,
                            ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                            TPM2_CAP_TPM_PROPERTIES, 0, TPM2_MAX_TPM_PROPERTIES,
                            NULL, &cprov->capability.properties);
-    TPM2_CHECK_RC(cprov->core, r, TPM2_ERR_CANNOT_GET_CAPABILITY, goto err3);
+    TPM2_CHECK_RC(cprov->core, r, TPM2_ERR_CANNOT_GET_CAPABILITY, goto err4);
 
     r = Esys_GetCapability(cprov->esys_ctx,
                            ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                            TPM2_CAP_ALGS, 0, TPM2_MAX_CAP_ALGS,
                            NULL, &cprov->capability.algorithms);
-    TPM2_CHECK_RC(cprov->core, r, TPM2_ERR_CANNOT_GET_CAPABILITY, goto err3);
+    TPM2_CHECK_RC(cprov->core, r, TPM2_ERR_CANNOT_GET_CAPABILITY, goto err4);
 
     r = Esys_GetCapability(cprov->esys_ctx,
                            ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                            TPM2_CAP_COMMANDS, 0, TPM2_MAX_CAP_CC,
                            NULL, &cprov->capability.commands);
-    TPM2_CHECK_RC(cprov->core, r, TPM2_ERR_CANNOT_GET_CAPABILITY, goto err3);
+    TPM2_CHECK_RC(cprov->core, r, TPM2_ERR_CANNOT_GET_CAPABILITY, goto err4);
 
     *out = tpm2_dispatch_table;
     *provctx = cprov;
 
     return 1;
+err4:
+    tpm2_semaphore_free(cprov->esys_lock);
 err3:
     Esys_Finalize(&cprov->esys_ctx);
 err2:
