@@ -31,16 +31,17 @@ DNS.1               = localhost
 EOF
 
 # create a RSAE private key and then generate a self-signed certificate for it
-openssl req -provider tpm2 -x509 -config testcert.conf -out testcert.pem
+openssl req -provider tpm2 -provider default -propquery '?provider=tpm2' \
+            -x509 -config testcert.conf -out testcert.pem
 
 # display content of the certificate
 openssl x509 -text -noout -in testcert.pem
 
-# start SSL server with RSA-PSS-RSAE signing
-openssl s_server -provider tpm2 -provider default \
-                 -accept 4443 -www -key testkey.pem -cert testcert.pem &
+# start SSL server with RSA-PSS-RSAE signing, port 4431
+openssl s_server -provider tpm2 -provider default -propquery '?provider=tpm2' \
+                 -accept 4431 -www -key testkey.pem -cert testcert.pem &
 SERVER=$!
 trap "cleanup" EXIT
 
 # start SSL client
-curl --retry 5 --retry-connrefused --cacert testcert.pem https://localhost:4443/
+curl --retry 5 --retry-connrefused --cacert testcert.pem https://localhost:4431/
